@@ -13,27 +13,26 @@
 // limitations under the License.
 
 //! Sync server of ttrpc.
+//!
+
+#[cfg(unix)]
+use std::os::unix::io::{AsRawFd, FromRawFd, RawFd};
 
-use nix::sys::socket::{self, *};
-use nix::unistd::*;
 use protobuf::{CodedInputStream, Message};
 use std::collections::HashMap;
-use std::os::unix::io::{AsRawFd, FromRawFd, RawFd};
 use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 use std::sync::mpsc::{channel, sync_channel, Receiver, Sender, SyncSender};
 use std::sync::{Arc, Mutex};
+use std::thread;
 use std::thread::JoinHandle;
-use std::{io, thread};
 
-#[cfg(not(target_os = "linux"))]
-use crate::common::set_fd_close_exec;
-use crate::common::{self, MESSAGE_TYPE_REQUEST};
+use super::utils::{response_error_to_channel, response_to_channel};
 use crate::context;
 use crate::error::{get_status, Error, Result};
+use crate::proto::{Code, MessageHeader, Request, Response, MESSAGE_TYPE_REQUEST};
 use crate::sync::channel::{read_message, write_message};
-use crate::ttrpc::{Code, Request, Response};
-use crate::MessageHeader;
-use crate::{response_to_channel, MethodHandler, TtrpcContext};
+use crate::sync::sys::{PipeConnection, PipeListener};
+use crate::{MethodHandler, TtrpcContext};
 
 // poll_queue will create WAIT_THREAD_COUNT_DEFAULT threads in begin.
 // If wait thread count < WAIT_THREAD_COUNT_MIN, create number to WAIT_THREAD_COUNT_DEFAULT.
@@ -44,13 +43,14 @@ const DEFAULT_WAIT_THREAD_COUNT_MAX: usize = 5;
 
 type MessageSender = Sender<(MessageHeader, Vec<u8>)>;
 type MessageReceiver = Receiver<(MessageHeader, Vec<u8>)>;
+type WorkloadSender = crossbeam::channel::Sender<(MessageHeader, Result<Vec<u8>>)>;
+type WorkloadReceiver = crossbeam::channel::Receiver<(MessageHeader, Result<Vec<u8>>)>;
 
 /// A ttrpc Server (sync).
 pub struct Server {
-    listeners: Vec<RawFd>,
-    monitor_fd: (RawFd, RawFd),
+    listeners: Vec<Arc<PipeListener>>,
     listener_quit_flag: Arc<AtomicBool>,
-    connections: Arc<Mutex<HashMap<RawFd, Connection>>>,
+    connections: Arc<Mutex<HashMap<i32, Connection>>>,
     methods: Arc<HashMap<String, Box<dyn MethodHandler + Send + Sync>>>,
     handler: Option<JoinHandle<()>>,
     reaper: Option<(Sender<i32>, JoinHandle<()>)>,
@@ -60,40 +60,48 @@ pub struct Server {
 }
 
 struct Connection {
-    fd: RawFd,
+    connection: Arc<PipeConnection>,
     quit: Arc<AtomicBool>,
     handler: Option<JoinHandle<()>>,
 }
 
 impl Connection {
     fn close(&self) {
+        self.connection.close().unwrap_or(());
+    }
+
+    fn shutdown(&self) {
         self.quit.store(true, Ordering::SeqCst);
+
         // in case the connection had closed
-        socket::shutdown(self.fd, Shutdown::Read).unwrap_or(());
+        self.connection.shutdown().unwrap_or(());
     }
 }
 
 struct ThreadS<'a> {
-    fd: RawFd,
-    fdlock: &'a Arc<Mutex<()>>,
+    connection: &'a Arc<PipeConnection>,
+    workload_rx: &'a WorkloadReceiver,
     wtc: &'a Arc<AtomicUsize>,
     quit: &'a Arc<AtomicBool>,
     methods: &'a Arc<HashMap<String, Box<dyn MethodHandler + Send + Sync>>>,
     res_tx: &'a MessageSender,
     control_tx: &'a SyncSender<()>,
+    cancel_rx: &'a crossbeam::channel::Receiver<()>,
     default: usize,
     min: usize,
     max: usize,
 }
 
+#[allow(clippy::too_many_arguments)]
 fn start_method_handler_thread(
-    fd: RawFd,
-    fdlock: Arc<Mutex<()>>,
+    connection: Arc<PipeConnection>,
+    workload_rx: WorkloadReceiver,
     wtc: Arc<AtomicUsize>,
     quit: Arc<AtomicBool>,
     methods: Arc<HashMap<String, Box<dyn MethodHandler + Send + Sync>>>,
     res_tx: MessageSender,
     control_tx: SyncSender<()>,
+    cancel_rx: crossbeam::channel::Receiver<()>,
     min: usize,
     max: usize,
 ) {
@@ -105,24 +113,13 @@ fn start_method_handler_thread(
                 break;
             }
 
-            let result;
-            {
-                let _guard = fdlock.lock().unwrap();
-                if quit.load(Ordering::SeqCst) {
-                    // notify the connection dealing main thread to stop.
-                    control_tx
-                        .send(())
-                        .unwrap_or_else(|err| debug!("Failed to send {:?}", err));
-                    break;
-                }
-                result = read_message(fd);
-            }
+            let result = workload_rx.recv();
 
             if quit.load(Ordering::SeqCst) {
                 // notify the connection dealing main thread to stop.
                 control_tx
                     .send(())
-                    .unwrap_or_else(|err| debug!("Failed to send {:?}", err));
+                    .unwrap_or_else(|err| trace!("Failed to send {:?}", err));
                 break;
             }
 
@@ -131,32 +128,30 @@ fn start_method_handler_thread(
                 trace!("notify client handler to create much more worker threads!");
                 control_tx
                     .send(())
-                    .unwrap_or_else(|err| debug!("Failed to send {:?}", err));
+                    .unwrap_or_else(|err| trace!("Failed to send {:?}", err));
             }
 
             let mh;
             let buf;
             match result {
-                Ok((x, y)) => {
+                Ok((x, Ok(y))) => {
                     mh = x;
                     buf = y;
                 }
-                Err(x) => match x {
-                    Error::Socket(y) => {
-                        trace!("Socket error {}", y);
-                        quit.store(true, Ordering::SeqCst);
-                        // the client connection would be closed and
-                        // the connection dealing main thread would
-                        // have exited.
-                        control_tx
-                            .send(())
-                            .unwrap_or_else(|err| debug!("Failed to send {:?}", err));
-                        trace!("Socket error send control_tx");
+                Ok((mh, Err(e))) => {
+                    if let Err(x) = response_error_to_channel(mh.stream_id, e, res_tx.clone()) {
+                        debug!("response_error_to_channel get error {:?}", x);
+                        quit_connection(quit, control_tx);
                         break;
                     }
-                    _ => {
-                        trace!("Others error {:?}", x);
-                        continue;
+                    continue;
+                }
+                Err(x) => match x {
+                    crossbeam::channel::RecvError => {
+                        trace!("workload_rx recv error");
+                        quit_connection(quit, control_tx);
+                        trace!("workload_rx recv error, send control_tx");
+                        break;
                     }
                 },
             }
@@ -172,13 +167,7 @@ fn start_method_handler_thread(
                 res.set_status(status);
                 if let Err(x) = response_to_channel(mh.stream_id, res, res_tx.clone()) {
                     debug!("response_to_channel get error {:?}", x);
-                    quit.store(true, Ordering::SeqCst);
-                    // the client connection would be closed and
-                    // the connection dealing main thread would have
-                    // exited.
-                    control_tx
-                        .send(())
-                        .unwrap_or_else(|err| debug!("Failed to send {:?}", err));
+                    quit_connection(quit, control_tx);
                     break;
                 }
                 continue;
@@ -186,28 +175,22 @@ fn start_method_handler_thread(
             trace!("Got Message request {:?}", req);
 
             let path = format!("/{}/{}", req.service, req.method);
-            let method;
-            if let Some(x) = methods.get(&path) {
-                method = x;
+            let method = if let Some(x) = methods.get(&path) {
+                x
             } else {
-                let status = get_status(Code::INVALID_ARGUMENT, format!("{} does not exist", path));
+                let status = get_status(Code::INVALID_ARGUMENT, format!("{path} does not exist"));
                 let mut res = Response::new();
                 res.set_status(status);
                 if let Err(x) = response_to_channel(mh.stream_id, res, res_tx.clone()) {
                     info!("response_to_channel get error {:?}", x);
-                    quit.store(true, Ordering::SeqCst);
-                    // the client connection would be closed and
-                    // the connection dealing main thread would have
-                    // exited.
-                    control_tx
-                        .send(())
-                        .unwrap_or_else(|err| debug!("Failed to send {:?}", err));
+                    quit_connection(quit, control_tx);
                     break;
                 }
                 continue;
-            }
+            };
             let ctx = TtrpcContext {
-                fd,
+                fd: connection.id(),
+                cancel_rx: cancel_rx.clone(),
                 mh,
                 res_tx: res_tx.clone(),
                 metadata: context::from_pb(&req.metadata),
@@ -215,13 +198,7 @@ fn start_method_handler_thread(
             };
             if let Err(x) = method.handler(ctx, req) {
                 debug!("method handle {} get error {:?}", path, x);
-                quit.store(true, Ordering::SeqCst);
-                // the client connection would be closed and
-                // the connection dealing main thread would have
-                // exited.
-                control_tx
-                    .send(())
-                    .unwrap_or_else(|err| debug!("Failed to send {:?}", err));
+                quit_connection(quit, control_tx);
                 break;
             }
         }
@@ -234,13 +211,14 @@ fn start_method_handler_threads(num: usize, ts: &ThreadS) {
             break;
         }
         start_method_handler_thread(
-            ts.fd,
-            ts.fdlock.clone(),
+            ts.connection.clone(),
+            ts.workload_rx.clone(),
             ts.wtc.clone(),
             ts.quit.clone(),
             ts.methods.clone(),
             ts.res_tx.clone(),
             ts.control_tx.clone(),
+            ts.cancel_rx.clone(),
             ts.min,
             ts.max,
         );
@@ -258,7 +236,6 @@ impl Default for Server {
     fn default() -> Self {
         Server {
             listeners: Vec::with_capacity(1),
-            monitor_fd: (-1, -1),
             listener_quit_flag: Arc::new(AtomicBool::new(false)),
             connections: Arc::new(Mutex::new(HashMap::new())),
             methods: Arc::new(HashMap::new()),
@@ -276,22 +253,30 @@ impl Server {
         Server::default()
     }
 
-    pub fn bind(mut self, host: &str) -> Result<Server> {
+    pub fn bind(mut self, sockaddr: &str) -> Result<Server> {
         if !self.listeners.is_empty() {
             return Err(Error::Others(
-                "ttrpc-rust just support 1 host now".to_string(),
+                "ttrpc-rust just support 1 sockaddr now".to_string(),
             ));
         }
 
-        let (fd, _) = common::do_bind(host)?;
-        common::do_listen(fd)?;
+        let listener = PipeListener::new(sockaddr)?;
 
-        self.listeners.push(fd);
+        self.listeners.push(Arc::new(listener));
         Ok(self)
     }
 
+    #[cfg(unix)]
     pub fn add_listener(mut self, fd: RawFd) -> Result<Server> {
-        self.listeners.push(fd);
+        if !self.listeners.is_empty() {
+            return Err(Error::Others(
+                "ttrpc-rust just support 1 sockaddr now".to_string(),
+            ));
+        }
+
+        let listener = PipeListener::new_from_fd(fd)?;
+
+        self.listeners.push(Arc::new(listener));
 
         Ok(self)
     }
@@ -329,27 +314,12 @@ impl Server {
 
         self.listener_quit_flag.store(false, Ordering::SeqCst);
 
-        #[cfg(target_os = "linux")]
-        let fds = pipe2(nix::fcntl::OFlag::O_CLOEXEC)?;
-
-        #[cfg(not(target_os = "linux"))]
-        let fds = {
-            let (rfd, wfd) = pipe()?;
-            set_fd_close_exec(rfd)?;
-            set_fd_close_exec(wfd)?;
-            (rfd, wfd)
-        };
-
-        self.monitor_fd = fds;
-
-        let listener = self.listeners[0];
-
+        let listener = self.listeners[0].clone();
         let methods = self.methods.clone();
         let default = self.thread_count_default;
         let min = self.thread_count_min;
         let max = self.thread_count_max;
         let listener_quit_flag = self.listener_quit_flag.clone();
-        let monitor_fd = self.monitor_fd.0;
 
         let reaper_tx = match self.reaper.take() {
             None => {
@@ -366,7 +336,7 @@ impl Server {
                                 .map(|mut cn| {
                                     cn.handler.take().map(|handler| {
                                         handler.join().unwrap();
-                                        close(fd).unwrap();
+                                        cn.close()
                                     })
                                 });
                         }
@@ -386,78 +356,15 @@ impl Server {
         let handler = thread::Builder::new()
             .name("listener_loop".into())
             .spawn(move || {
-                let mut pollers = vec![
-                    libc::pollfd {
-                        fd: monitor_fd,
-                        events: libc::POLLIN,
-                        revents: 0,
-                    },
-                    libc::pollfd {
-                        fd: listener,
-                        events: libc::POLLIN,
-                        revents: 0,
-                    },
-                ];
-
                 loop {
-                    if listener_quit_flag.load(Ordering::SeqCst) {
-                        info!("listener shutdown for quit flag");
-                        break;
-                    }
-
-                    let returned = unsafe {
-                        let pollers: &mut [libc::pollfd] = &mut pollers;
-                        libc::poll(
-                            pollers as *mut _ as *mut libc::pollfd,
-                            pollers.len() as _,
-                            -1,
-                        )
-                    };
-
-                    if returned == -1 {
-                        let err = io::Error::last_os_error();
-                        if err.raw_os_error() == Some(libc::EINTR) {
+                    trace!("listening...");
+                    let pipe_connection = match listener.accept(&listener_quit_flag) {
+                        Ok(None) => {
                             continue;
                         }
-
-                        error!("fatal error in listener_loop:{:?}", err);
-                        break;
-                    } else if returned < 1 {
-                        continue;
-                    }
-
-                    if pollers[0].revents != 0 || pollers[pollers.len() - 1].revents == 0 {
-                        continue;
-                    }
-
-                    if listener_quit_flag.load(Ordering::SeqCst) {
-                        info!("listener shutdown for quit flag");
-                        break;
-                    }
-
-                    #[cfg(target_os = "linux")]
-                    let fd = match accept4(listener, SockFlag::SOCK_CLOEXEC) {
-                        Ok(fd) => fd,
+                        Ok(Some(conn)) => Arc::new(conn),
                         Err(e) => {
-                            error!("failed to accept error {:?}", e);
-                            break;
-                        }
-                    };
-
-                    // Non Linux platforms do not support accept4 with SOCK_CLOEXEC flag, so instead
-                    // use accept and call fcntl separately to set SOCK_CLOEXEC.
-                    // Because of this there is chance of the descriptor leak if fork + exec happens in between.
-                    #[cfg(not(target_os = "linux"))]
-                    let fd = match accept(listener) {
-                        Ok(fd) => {
-                            if let Err(err) = set_fd_close_exec(fd) {
-                                error!("fcntl failed after accept: {:?}", err);
-                                break;
-                            };
-                            fd
-                        }
-                        Err(e) => {
-                            error!("failed to accept error {:?}", e);
+                            error!("listener accept got {:?}", e);
                             break;
                         }
                     };
@@ -466,6 +373,7 @@ impl Server {
                     let quit = Arc::new(AtomicBool::new(false));
                     let child_quit = quit.clone();
                     let reaper_tx_child = reaper_tx.clone();
+                    let pipe_connection_child = pipe_connection.clone();
 
                     let handler = thread::Builder::new()
                         .name("client_handler".into())
@@ -473,11 +381,12 @@ impl Server {
                             debug!("Got new client");
                             // Start response thread
                             let quit_res = child_quit.clone();
+                            let pipe = pipe_connection_child.clone();
                             let (res_tx, res_rx): (MessageSender, MessageReceiver) = channel();
                             let handler = thread::spawn(move || {
                                 for r in res_rx.iter() {
                                     trace!("response thread get {:?}", r);
-                                    if let Err(e) = write_message(fd, r.0, r.1) {
+                                    if let Err(e) = write_message(&pipe, r.0, r.1) {
                                         error!("write_message got {:?}", e);
                                         quit_res.store(true, Ordering::SeqCst);
                                         break;
@@ -489,13 +398,66 @@ impl Server {
 
                             let (control_tx, control_rx): (SyncSender<()>, Receiver<()>) =
                                 sync_channel(0);
+
+                            // start read message thread
+                            let quit_reader = child_quit.clone();
+                            let pipe_reader = pipe_connection_child.clone();
+                            let (workload_tx, workload_rx): (WorkloadSender, WorkloadReceiver) =
+                                crossbeam::channel::unbounded();
+                            let (cancel_tx, cancel_rx) = crossbeam::channel::unbounded::<()>();
+                            let control_tx_reader = control_tx.clone();
+                            let reader = thread::spawn(move || {
+                                while !quit_reader.load(Ordering::SeqCst) {
+                                    let msg = read_message(&pipe_reader);
+                                    match msg {
+                                        Ok((x, y)) => {
+                                            let res = workload_tx.send((x, y));
+                                            match res {
+                                                Ok(_) => {}
+                                                Err(crossbeam::channel::SendError(e)) => {
+                                                    error!("Send workload error {:?}", e);
+                                                    quit_reader.store(true, Ordering::SeqCst);
+                                                    control_tx_reader.send(()).unwrap_or_else(
+                                                        |err| trace!("Failed to send {:?}", err),
+                                                    );
+                                                    break;
+                                                }
+                                            }
+                                        }
+                                        Err(x) => match x {
+                                            Error::Socket(y) => {
+                                                trace!("Socket error {}", y);
+                                                drop(cancel_tx);
+                                                quit_reader.store(true, Ordering::SeqCst);
+                                                // the client connection would be closed and
+                                                // the connection dealing main thread would
+                                                // have exited.
+                                                control_tx_reader.send(()).unwrap_or_else(|err| {
+                                                    trace!("Failed to send {:?}", err)
+                                                });
+                                                trace!("Socket error send control_tx");
+                                                break;
+                                            }
+                                            _ => {
+                                                trace!("Other error {:?}", x);
+                                                continue;
+                                            }
+                                        },
+                                    }
+                                }
+
+                                trace!("read message thread quit");
+                            });
+
+                            let pipe = pipe_connection_child.clone();
                             let ts = ThreadS {
-                                fd,
-                                fdlock: &Arc::new(Mutex::new(())),
+                                connection: &pipe,
+                                workload_rx: &workload_rx,
                                 wtc: &Arc::new(AtomicUsize::new(0)),
                                 methods: &methods,
                                 res_tx: &res_tx,
                                 control_tx: &control_tx,
+                                cancel_rx: &cancel_rx,
                                 quit: &child_quit,
                                 default,
                                 min,
@@ -514,20 +476,24 @@ impl Server {
                             drop(control_rx);
                             // drop the res_tx, thus the res_rx would get terminated notification.
                             drop(res_tx);
+                            drop(workload_rx);
                             handler.join().unwrap_or(());
+                            reader.join().unwrap_or(());
                             // client_handler should not close fd before exit
                             // , which prevent fd reuse issue.
-                            reaper_tx_child.send(fd).unwrap();
+                            reaper_tx_child.send(pipe.id()).unwrap();
 
                             debug!("client thread quit");
                         })
                         .unwrap();
 
                     let mut cns = connections.lock().unwrap();
+
+                    let id = pipe_connection.id();
                     cns.insert(
-                        fd,
+                        id,
                         Connection {
-                            fd,
+                            connection: pipe_connection,
                             handler: Some(handler),
                             quit: quit.clone(),
                         },
@@ -553,7 +519,7 @@ impl Server {
         }
         if self.thread_count_default <= self.thread_count_min {
             return Err(Error::Others(
-                "thread_count_default should biger than thread_count_min".to_string(),
+                "thread_count_default should bigger than thread_count_min".to_string(),
             ));
         }
         self.start_listen()?;
@@ -563,12 +529,11 @@ impl Server {
 
     pub fn stop_listen(mut self) -> Self {
         self.listener_quit_flag.store(true, Ordering::SeqCst);
-        close(self.monitor_fd.1).unwrap_or_else(|e| {
-            warn!(
-                "failed to close notify fd: {} with error: {}",
-                self.monitor_fd.1, e
-            )
-        });
+
+        self.listeners[0]
+            .close()
+            .unwrap_or_else(|e| warn!("failed to close connection with error: {}", e));
+
         info!("close monitor");
         if let Some(handler) = self.handler.take() {
             handler.join().unwrap();
@@ -582,7 +547,7 @@ impl Server {
         let connections = self.connections.lock().unwrap();
 
         for (_fd, c) in connections.iter() {
-            c.close();
+            c.shutdown();
         }
         // release connections's lock, since the following handler.join()
         // would wait on the other thread's exit in which would take the lock.
@@ -601,14 +566,26 @@ impl Server {
     }
 }
 
+#[cfg(unix)]
 impl FromRawFd for Server {
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
         Self::default().add_listener(fd).unwrap()
     }
 }
 
+#[cfg(unix)]
 impl AsRawFd for Server {
     fn as_raw_fd(&self) -> RawFd {
-        self.listeners[0]
+        self.listeners[0].as_raw_fd()
     }
 }
+
+fn quit_connection(quit: Arc<AtomicBool>, control_tx: SyncSender<()>) {
+    quit.store(true, Ordering::SeqCst);
+    // the client connection would be closed and
+    // the connection dealing main thread would
+    // have exited.
+    control_tx
+        .send(())
+        .unwrap_or_else(|err| debug!("Failed to send {:?}", err));
+}
